
#include	<pic18.h>
#include 	<string.h>
#include	<ctype.h>
#include	<math.h>
#include	<stdlib.h>
#include	<stdio.h>
#include	"defines.h"                                   // IO- und Bitkonstanten

#define HalfStep    TRUE
#define	Drehpause			20

volatile unsigned long Ticker;                          // Systemuhr
volatile char trigger;
char Step=0;                                            // Aktueller Vollschritt (0..3)

// Schalterzustand
char Schalter;
char AltSchalter;

int Durchlauf;
int Startpause;
int MaxSchritt;
int Schrittzaehler;

int BlinkTicker = 0;
char BlinkFlag = FALSE;

//-------------------------------------------------
// Konfiguration des Controllers
//-------------------------------------------------

__CONFIG(1, OSCSEN & HS);                    // HS = Quarz
// __CONFIG(2, BORV42 & WDTDIS);      // Kein Watchdog, BrownOut 4.2V
__CONFIG(2, BORV42 & WDTPS128 & WDTEN);      // Kein Watchdog, BrownOut 4.2V
__CONFIG(4, LVPDIS & STVREN);                // Kein LowVoltageProgramm, Reset bei StackOverflow

/*********************
 *
 * Globale Variablen
 *
 *********************/
interrupt void INT_(void)
 {                                                   // FBD7
   //--------------------
   // Timer 1 berlauf ?
   //--------------------
   if (TMR1IF)
   	{
     	Ticker++;                                         // Systemticker erhhen
     	TMR1IF=0;                                         // Clear Interrupt-Flag
     	TMR1L=0x20;                                       // Timer 1 neu vorladen
     	TMR1H=0xFF;

      BlinkTicker++;
      if (BlinkTicker == 300)
       {
        BlinkTicker = 0;
        if (BlinkFlag) BlinkFlag = FALSE; else BlinkFlag = TRUE;
       }

     	/*
     	if (trigger)
     	 {
     	  trigger = FALSE;
     	  RC0 = 0;
     	 } else
     	 {
     	  trigger = TRUE;
     	  RC0 = 1;
     	 }
      */

   	} // From Timer 1 berlauf

   return;									                            // Zurck zum Hauptprogramm
 }

/*************************************
 *
 * Status des Wahlschalters reinholen
 *
 *************************************/
char GetSchalter (void)
 {
  if (PORTB & 1) return (POEL); else return (DIESEL);
 }

/*********************************
 *
 * Status des Sensors Plposition
 *
 *********************************/
char GetSensorPoel (void)
 {
  if (PORTB & 2) return (POEL); else return (DIESEL);
 }

/************************************
 *
 * Status des Sensors Dieselposition
 *
 ************************************/
char GetSensorDiesel (void)
 {
  if (PORTB & 4) return (POEL); else return (DIESEL);
 }

/**********************
 *
 * Mach mal Pause.....
 *
 **********************/
void Delayms (int time)
 {
  long ticker;

  ticker = Ticker;
  while ( (ticker + (time / 10 )) > Ticker ) CLRWDT();
 }

/*****************************
 *
 * Globale Initialisierungen
 *
 *****************************/
void Init (void)
 {
   PORTA=0;
   PORTB=0;
   PORTC=0;
   TRISA=32;
   TRISB=0xFF;                                                    // Alles Eingang
   TRISC=0;                                                       // Alles Ausgang

   RC0=0;                                                         // Lampe aus

   Ticker = 0;
   Step = 0;

   Durchlauf = 1000;
   Startpause = 50;

   TMR1IE=1;									                                    // enable Interrupt of TMR1
   T1CON=49;									                                    // Prescaler=1:8 + Timer on (49)

   //alle Interrupts an
   PEIE=1;          							                	              // Peripherie Int=on
   GIE=1;										                                      // Define Interupts

   // Aktuellen Schalterzustand prfen
   Schalter = GetSchalter ();
   AltSchalter = Schalter;

   // Wie weit wirst du heute gehen ?
   Schrittzaehler = 0;

   RC0=0;        // Lampe aus

   if (!HalfStep) MaxSchritt = DefMaxSchritt; else MaxSchritt = 2*DefMaxSchritt;
 }

/******************************
 *
 * Stepper Ein- und abschalten
 *
 ******************************/
void MotorAus (void)
 {
  EnB = 0;
  EnA = 0;
  RC0 = 0;   // Lampe aus
 }

/******************************************************
 *
 * Einen Vollschritt in die jeweilige Richtung machen
 *
 ******************************************************/
void NextStep (char Art)
 {
  if (BlinkFlag) RC0 = 1; else RC0 = 0;  // Lampe blinkt whrend Motor luft

  // Halbschrittbetrieb
  if (HalfStep)
   {
     // Nchste Postion errechnen
     if (Art) { Step++; if (Step == 8) Step = 0; } else
              { Step--; if (Step >  8) Step = 7; }

     // Schritt ausfhren
     if (Step == 0)
      {
       // Zweig A
       A1 = 1;
       A2 = 0;
       EnA = 1;

       // Zweig B
       B1 = 0;
       B2 = 0;
       EnB = 0;
      } else

     if (Step == 1)
      {
       // Zweig A
       A1 = 1;
       A2 = 0;
       EnA = 1;

       // Zweig B
       B1 = 1;
       B2 = 0;
       EnB = 1;
      } else

     if (Step == 2)
      {
       // Zweig A
       A1 = 0;
       A2 = 0;
       EnA = 0;

       // Zweig B
       B1 = 1;
       B2 = 0;
       EnB = 1;
      } else

     if (Step == 3)
      {
       // Zweig A
       A1 = 0;
       A2 = 1;
       EnA = 1;

       // Zweig B
       B1 = 1;
       B2 = 0;
       EnB = 1;
      } else

     if (Step == 4)
      {
       // Zweig A
       A1 = 0;
       A2 = 1;
       EnA = 1;

       // Zweig B
       B1 = 0;
       B2 = 0;
       EnB = 0;
      } else

     if (Step == 5)
      {
       // Zweig A
       A1 = 0;
       A2 = 1;
       EnA = 1;

       // Zweig B
       B1 = 0;
       B2 = 1;
       EnB = 1;
      } else

     if (Step == 6)
      {
       // Zweig A
       A1 = 0;
       A2 = 0;
       EnA = 0;

       // Zweig B
       B1 = 0;
       B2 = 1;
       EnB = 1;
      } else

     if (Step == 7)
      {
       // Zweig A
       A1 = 1;
       A2 = 0;
       EnA = 1;

       // Zweig B
       B1 = 0;
       B2 = 1;
       EnB = 1;
      }

   } else // Von Halbschrittbetrieb

   { // Vollschrittbetrieb


     // Nchste Postion errechnen
     if (Art) { Step++; if (Step == 4) Step = 0; } else
              { Step--; if (Step >  4) Step = 3; }

     // Schritt ausfhren
     if (Step == 0)
      {
       // Zweig A
       A1 = 1;
       A2 = 0;
       EnA = 1;

       // Zweig B
       B1 = 1;
       B2 = 0;
       EnB = 1;
      } else

     if (Step == 1)
      {
       // Zweig A
       A1 = 0;
       A2 = 1;
       EnA = 1;

       // Zweig B
       B1 = 1;
       B2 = 0;
       EnB = 1;
      } else

     if (Step == 2)
      {
       // Zweig A
       A1 = 0;
       A2 = 1;
       EnA = 1;

       // Zweig B
       B1 = 0;
       B2 = 1;
       EnB = 1;
      } else

     if (Step == 3)
      {
       // Zweig A
       A1 = 1;
       A2 = 0;
       EnA = 1;

       // Zweig B
       B1 = 0;
       B2 = 1;
       EnB = 1;
      }

   }

 }

/******************
 *
 * Fehler anzeigen
 *
 ******************/
void Fehler (void)
 {
  MotorAus ();
  RC0=1;                  // Lampe dauernd an

  Schalter = GetSchalter ();
  AltSchalter = Schalter;

  while (Schalter == AltSchalter)
   {
    Schalter = GetSchalter ();
    CLRWDT();
   }

 }

/******************
 *
 * Motor verfahren
 *
 ******************/
void Stepper (void)
 {

   //-------------------------------------------------------------------------
   // Erste berprfung, ob Schalterposition mit Sensorposition bereinstimmt
   //-------------------------------------------------------------------------

   // Pl gewnscht, aber nicht in der passenden Position ? Dann nix wie los
   if ( (GetSchalter () == POEL) && (!(GetSensorPoel ())) )
    {

     Schrittzaehler = 0;
     while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorPoel ())) )
      {
       NextStep (DIESEL);
       Delayms (Drehpause);
       Schrittzaehler++;
      } // Von while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorPoel ()) )

     // Ist die Position evtl. nicht erreicht worden ?
     MotorAus ();
     if (Schrittzaehler >= MaxSchritt) Fehler ();

    } else

   // Diesel gewnscht, aber nicht in der passenden Position ? Dann nix wie los
   if ( (GetSchalter () == DIESEL) && (!(GetSensorDiesel ())) )
    {

     Schrittzaehler = 0;
     while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorDiesel ())) )
      {
       NextStep (POEL);
       Delayms (Drehpause);
       Schrittzaehler++;
      } // Von while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorPoel ()) )

     // Ist die Position evtl. nicht erreicht worden ?
     MotorAus ();
     if (Schrittzaehler >= MaxSchritt) Fehler ();

    }

   // Hier gelandet ist wohl alles gut im Staate.....
   MotorAus ();

   //----------------------------
   // Warten auf Aktionskommando
   //----------------------------
   while (1)
    {

      MotorAus ();
      CLRWDT();
      Schalter = GetSchalter ();
      if (Schalter != AltSchalter)
       { // Schalter wurde umgelegt, also neue Position anfahren

         // Pl gewnscht, aber nicht in der passenden Position ? Dann nix wie los
         if ( (Schalter == POEL) && (!(GetSensorPoel ())) )
          {

           Schrittzaehler = 0;
           while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorPoel ())) )
            {
             NextStep (DIESEL);
             Delayms (Drehpause);
             Schrittzaehler++;
            } // Von while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorPoel ()) )

           // Ist die Position evtl. nicht erreicht worden ?
           if (Schrittzaehler >= MaxSchritt) Fehler ();

          }

         // Diesel gewnscht, aber nicht in der passenden Position ? Dann nix wie los
         if ( (Schalter == DIESEL) && (!(GetSensorDiesel ())) )
          {

           Schrittzaehler = 0;
           while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorDiesel ())) )
            {
             NextStep (POEL);
             Delayms (Drehpause);
             Schrittzaehler++;
            } // Von while ( (Schrittzaehler < MaxSchritt) && (!(GetSensorPoel ()) )

           // Ist die Position evtl. nicht erreicht worden ?
           if (Schrittzaehler >= MaxSchritt) Fehler ();

          }

        // Position festhalten
        AltSchalter = Schalter;
       } // Von if (Schalter != AltSchalter)


    } // Von while (1)
 }

/********
 *
 * Main
 *
 ********/
void main (void)
 {
  Init ();								            // Initialisieren
  Stepper ();                         // Motor laufen lassen
 }

